@@ -12,8 +12,62 @@ use uuid::Uuid;
 
 const RAW_STATE_CACHE_MAX_CAPACITY: usize = 1_000_000;
 
+#[derive(Clone)]
+pub struct RawStateEventPreparedOwned {
+    state: RawStateEventOwned,
+    delta: Option<f64>,
+}
+
+impl RawStateEventPreparedOwned {
+    #[inline]
+    pub fn from_rse_owned(state: RawStateEventOwned, delta: Option<f64>) -> Self {
+        Self { state, delta }
+    }
+    #[inline]
+    pub fn delta(&self) -> Option<f64> {
+        self.delta
+    }
+    #[inline]
+    pub fn state(&self) -> &RawStateEventOwned {
+        &self.state
+    }
+    #[inline]
+    pub fn state_mut(&mut self) -> &mut RawStateEventOwned {
+        &mut self.state
+    }
+    pub fn is_modified(&self, prev: &RawStateEventOwned) -> bool {
+        if !self.state.force && self.state.status == prev.status {
+            if self.state.value == prev.value {
+                return false;
+            }
+            if let Some(delta_v) = self.delta {
+                if let ValueOptionOwned::Value(ref prev_value) = prev.value {
+                    if let ValueOptionOwned::Value(ref current_value) = self.state.value {
+                        if let Ok(prev_value_f) = TryInto::<f64>::try_into(prev_value.clone()) {
+                            if let Ok(current_value_f) =
+                                TryInto::<f64>::try_into(current_value.clone())
+                            {
+                                if (current_value_f - prev_value_f).abs() < delta_v {
+                                    return false;
+                                }
+                            }
+                        }
+                    }
+                }
+            }
+        }
+        true
+    }
+}
+
+impl From<RawStateEventPreparedOwned> for RawStateEventOwned {
+    fn from(s: RawStateEventPreparedOwned) -> Self {
+        s.state
+    }
+}
+
 pub struct RawStateCache {
-    cache: Mutex<TtlCache<OID, RawStateEventOwned>>,
+    cache: Mutex<TtlCache<OID, RawStateEventPreparedOwned>>,
     ttl: Option<Duration>,
 }
 
@@ -30,15 +84,24 @@ impl RawStateCache {
     /// # Panics
     ///
     /// Will panic if the mutex is poisoned
-    pub fn push_check(&self, oid: &OID, raw_state: &RawStateEventOwned) -> bool {
+    pub fn push_check(
+        &self,
+        oid: &OID,
+        raw_state: &RawStateEventOwned,
+        delta: Option<f64>,
+    ) -> bool {
         if let Some(ttl) = self.ttl {
             let mut cache = self.cache.lock().unwrap();
             if let Some(v) = cache.get(oid) {
-                if raw_state == v {
+                if !v.is_modified(raw_state) {
                     return false;
                 }
             }
-            cache.insert(oid.clone(), raw_state.clone(), ttl);
+            cache.insert(
+                oid.clone(),
+                RawStateEventPreparedOwned::from_rse_owned(raw_state.clone(), delta),
+                ttl,
+            );
             true
         } else {
             false
@@ -50,12 +113,12 @@ impl RawStateCache {
     /// # Panics
     ///
     /// Will panic if the mutex is poisoned
-    pub fn retain_map_modified(&self, states: &mut HashMap<&OID, RawStateEventOwned>) {
+    pub fn retain_map_modified(&self, states: &mut HashMap<&OID, RawStateEventPreparedOwned>) {
         if let Some(ttl) = self.ttl {
             let mut cache = self.cache.lock().unwrap();
             states.retain(|oid, raw| {
                 if let Some(cached) = cache.get(oid) {
-                    cached != raw
+                    cached.is_modified(raw.state())
                 } else {
                     true
                 }
@@ -98,7 +161,7 @@ pub enum ItemProp {
 pub struct Action {
     uuid: Uuid,
     i: OID,
-    #[serde(deserialize_with = "eva_common::tools::deserialize_duration_from_nanos")]
+    #[serde(deserialize_with = "eva_common::tools::deserialize_duration_from_micros")]
     timeout: Duration,
     priority: u8,
     params: Option<actions::Params>,
